# frozen_string_literal: true

module Canistor
  module Storage
    # Represents the non-existence of an object on a versioned bucket.
    class DeleteMarker
      attr_reader :version_id

      def initialize
        @version_id = generate_version_id
      end

      def headers
        {
          'x-amz-delete-marker' => 'true',
          'x-amz-version-id' => version_id
        }
      end

      def head(context, subject)
        Canistor::ErrorHandler.serve_no_such_key(context, subject)
      end

      def get(context, subject)
        Canistor::ErrorHandler.serve_no_such_key(context, subject)
      end

      def delete(context, subject)
        context.http_response.signal_headers(200, headers)
      end

      private

      def generate_version_id
        Base64.strict_encode64(SecureRandom.hex(16)).gsub('=', '')
      end
    end
  end
end
