# frozen_string_literal: true

module Canistor
  module Storage
    # Manages a lookup table of versioned objects.
    class Objects
      def initialize(versioned: true)
        @versioned = versioned
        @objects = {}
      end

      def versioned?
        @versioned
      end

      def [](key)
        if stack = @objects[key]
          stack.last
        end
      end

      def []=(key, object)
        @objects[key] ||= []
        if versioned?
          @objects[key].push(object)
        else
          @objects[key] = [object]
        end
      end

      def delete(key)
        if versioned?
          self[key] = Canistor::Storage::DeleteMarker.new
        else
          @objects.delete(key).last
        end
      end

      def dig(*segments)
        @objects.dig(*segments)&.last
      end

      def each
        @objects.each do |key, stack|
          yield [key, stack.last]
        end
      end
    end
  end
end
