# frozen_string_literal: true

module Canistor
  module Storage
    class Part
      attr_accessor :region
      attr_accessor :bucket
      attr_accessor :key
      attr_accessor :upload_id
      attr_accessor :number

      attr_reader :data

      def initialize(**attributes)
        attributes.each do |name, value|
          send("#{name}=", value)
        end
      end

      def put(context, subject)
        write(context.http_request.body.read)
        context.http_response.signal_headers(200, identity_headers)
      end

      def etag
        '"' + digest + '"'
      end

      def size
        data&.size
      end

      def attributes
        {
          region: region,
          bucket: bucket,
          key: key,
          upload_id: upload_id,
          number: number,
          digest: digest,
          etag: etag
        }
      end

    private

      attr_writer :data

      def write(data)
        self.data = data
      end

      def digest
        @digest ||= Digest::SHA1.hexdigest(data)
      end

      def identity_headers
        {
          'x-amz-request-id' => Base64.strict_encode64(SecureRandom.hex(16)),
          'x-amz-id' => digest[0, 16],
          'x-amz-id-2' => Base64.strict_encode64(digest),
          'etag' => etag
        }
      end
    end
  end
end
