# frozen_string_literal: true

module Canistor
  class Subject
    attr_reader :uri

    attr_reader :region
    attr_reader :bucket
    attr_reader :key

    HOST_RE = /\A([^\.]+\.)?(s3\.([^\.]+)|s3-([^\.]+))/

    def initialize(uri)
      @uri = uri

      host_segments = HOST_RE.match(uri.host)
      path_segments = uri.path.split('/', 3)[1..-1] || []

      @region = parse_region(host_segments)
      @bucket = parse_bucket(host_segments, path_segments)
      @key = path_segments.empty? ? nil : path_segments.join('/')
    end

    private

    def parse_region(host_segments)
      case host_segments[3]
      when nil
        host_segments[4]
      when 'amazonaws'
        'us-east-1'
      else
        host_segments[3]
      end
    end

    def parse_bucket(host_segments, path_segments)
      if host_segments[1]
        host_segments[1][0..-2]
      else
        path_segments.shift
      end
    end
  end
end
